local moduleName = "Basics"
local core = SimpleUnitFrames

local Basics = core:NewModule(moduleName, "AceEvent-2.0")
local DEFAULT_FONT = "GameFontHighlightSmall"
local DEFAULT_FONT_SIZE = 10
local creatureTypeMap = core.loc.UnitCreatureType

-- List of fonts to be shown in the configuration.
local fonts = {
	"GameFontHighlight",
	"GameFontHighlightSmall",
	"GameFontNormal",
	"GameFontNormalSmall",
	"TextStatusBarText",
}

local inVehicle = {}

-- List of text value functions.
local staticStyles = {
	-- Health Styles
	["HPcurrent"] = "[HP:Short]",
	["HPdeficit"] = "[0-MissingHP]",
	["HPpercent"] = "[PercentHP:Percent]",
	["HPcurrmax"] = "[FractionalHP:Short]",
	["HPcomplete"] = "[FractionalHP:Short] [PercentHP:Percent:Paren]",
	-- Mana Styles
	["MPcurrent"] = "[HasMP ? MP ! \"-\"]",
	["MPdeficit"] = "[HasMP ? 0-MissingMP]",
	["MPpercent"] = "[HasMP ? PercentMP:Percent ! \"-\"]",
	["MPcurrmax"] = "[HasMP ? FractionalMP:Short ! \"-\"]",
	["MPcomplete"] = "[HasMP ? FractionalMP:Short \" \" PercentMP:Percent:Paren ! \"-\"]",
	-- No Style
	["none"] = "",
}

local unitMap = {}

local icons = {
	DEATHKNIGHT = "DEATHKNIGHT",
	DRUID = "DRUID",
	HUNTER = "HUNTER",
	MAGE = "MAGE",
	PALADIN = "PALADIN",
	PRIEST = "PRIEST",
	ROGUE = "ROGUE",
	SHAMAN = "SHAMAN",
	UNKNOWN = "UNKNOWN",
	WARLOCK = "WARLOCK",
	WARRIOR = "WARRIOR",
}

local frames = {}

Basics.formatList = {}
for k in pairs(staticStyles) do
	table.insert(Basics.formatList, k)
	table.sort(Basics.formatList)
end

Basics.unitMap = unitMap

Basics.consoleOptions = {}
Basics.defaultOptions = {} 
Basics.frameSettings = {}

function Basics:OnInitialize()
	self.db = core:AcquireDBNamespace(moduleName)

	core:RegisterDefaults(moduleName, 'profile', self.defaultOptions)
	
	for english, locale in pairs(creatureTypeMap) do
		icons[locale] = english
	end
	creatureTypeMap = nil
	
	local args = self.consoleOptions
	
	local settings = self.frameSettings

	self.frames = frames
	for unit, unitSettings in pairs(settings) do
		local frame = CreateFrame("Frame", nil, unitSettings.parent)
		frame:SetFrameLevel(unitSettings.parent:GetFrameLevel()+2)
		frame.fontStrings = {}
		frames[unit] = frame
		for text, textSettings in pairs(unitSettings.texts) do
			local mappedUnit = unitMap[unit] or unit
			local fontString = frame:CreateFontString(nil, "OVERLAY", (self.db.profile[mappedUnit] and self.db.profile[mappedUnit].font) or DEFAULT_FONT )
			fontString:SetPoint(unpack(textSettings.point))
			frame.fontStrings[text] = fontString
		end
		if unitSettings.classIcon then
			local texture = frame:CreateTexture("OVERLAY")
			texture:SetHeight(unitSettings.classIcon.size)
			texture:SetWidth(unitSettings.classIcon.size)
			texture:SetPoint( unpack(unitSettings.classIcon.point ) )
			frame.classIcon = texture
		end
	end

	for unit, options in pairs(args) do
		core.consoleOptions.args[unit] = options
		if unit ~= "style" then
			options.args.font = {
				order = 1000,
				type = 'text',
				name = 'Font',
				desc = 'Set the font for this frame.',
				get = function() return self.db.profile[unit].font or DEFAULT_FONT end,
				set = function(v)
					self.db.profile[unit].font = v
					self:RefreshFont(unit)
				end,
				validate = fonts
			}
		end
	end
	self.consoleOptions = nil
	self.defaultOptions = nil
	self.frameSettings = nil
end

function Basics:OnEnable()
	self:RegisterEvent("UNIT_HEALTH", "Refresh")
	self:RegisterEvent("UNIT_MAXHEALTH", "Refresh")
	self:RegisterEvent("UNIT_MANA", "Refresh")
	self:RegisterEvent("UNIT_MAXMANA", "Refresh")
	self:RegisterEvent("UNIT_RAGE", "Refresh")
	self:RegisterEvent("UNIT_MAXRAGE", "Refresh")
	self:RegisterEvent("UNIT_ENERGY", "Refresh")
	self:RegisterEvent("UNIT_MAXENERGY", "Refresh")
	self:RegisterEvent("UNIT_DISPLAYPOWER", "Refresh")
	self:RegisterEvent("UNIT_ENTERING_VEHICLE")
	self:RegisterEvent("UNIT_ENTERED_VEHICLE", "Refresh")
	self:RegisterEvent("UNIT_EXITING_VEHICLE")
	self:RegisterEvent("UNIT_EXITED_VEHICLE", "Refresh")
	self:Refresh()
end

function Basics:OnDisable()
	self:UnregisterAllEvents()
	for _, unitFrame in pairs(frames) do
		if unitFrame.classIcon then
			unitFrame.classIcon:Hide()
		end
		if unitFrame.fontStrings then
			for text, fontString in pairs(unitFrame.fontStrings) do
				fontString:SetText("")
			end
		end
	end
end

function Basics:UNIT_ENTERING_VEHICLE(unit, vehicle)
	inVehicle[unit] = vehicle
	if inVehicle[unit] == true then
		if unit == "player" then
			frames["player"], frames["pet"] = frames["pet"], frames["player"]
			unitMap["player"], unitMap["pet"] = "pet", "player"
		elseif unit:find("^party[1-4]$") then
			local pet = unit:gsub("party", "partypet")
			frames[unit], frames[pet] = frames[pet], frames[unit]
			unitMap[unit], unitMap[pet] = "partypet", "party"
		end
	end
	self:Refresh()
end

function Basics:UNIT_EXITING_VEHICLE(unit)
	if inVehicle[unit] == true then
		if unit == "player" then
			frames["player"], frames["pet"] = frames["pet"], frames["player"]
			unitMap["player"], unitMap["pet"] = nil, nil
		elseif unit:find("^party[1-4]$") then
			local pet = unit:gsub("party", "partypet")
			frames[unit], frames[pet] = frames[pet], frames[unit]
			unitMap[unit], unitMap[pet] = "party", "partypet"
		end
	end
	inVehicle[unit] = nil
	self:Refresh()
end

function Basics:Refresh(unit)
	if unit then
		if UnitExists(unit) then
			local mappedUnit = unitMap[unit] or unit
			if mappedUnit and frames[unit] then
				if frames[unit].classIcon then
					self:UpdateClassIcon(unit)
				end
				for text, fontString in pairs(frames[unit].fontStrings) do
					self:UpdateValue(unit, self.db.profile[mappedUnit][text], fontString )
				end
			end
		end
	else
		for unit in pairs(frames) do
			self:Refresh(unit)
		end
	end
end

function Basics:RefreshFont(unit)
	if unit then
		for frameUnit in pairs(frames) do
			if( frameUnit:find('^'..unit) ) then
				for text, fontString in pairs(frames[frameUnit].fontStrings) do
					local font = self.db.profile[unit].font or DEFAULT_FONT
					fontString:SetFontObject(font)
				end
			end
		end
	end
end

function Basics:UpdateValue(unit, style, fontString)
	local text = "ERROR"
	if staticStyles[style] then 
		text = LibStub("LibDogTag-3.0"):Evaluate(staticStyles[style], "Unit", {["unit"] = unit})
	elseif self.db.profile.style[style] then
		text = LibStub("LibDogTag-3.0"):Evaluate(self.db.profile.style[style], "Unit", {["unit"] = unit})
	end
	fontString:SetText(text)
end

function Basics:UpdateClassIcon(unit)
	local classIcon = frames[unit] and frames[unit].classIcon
	local mappedUnit = unitMap[unit] or unit
	
	if classIcon and mappedUnit and UnitExists(unit) then
	
		if self.db.profile[mappedUnit].class then
		
			local icon = nil
			if UnitIsPlayer(unit) then
				_, icon = UnitClass(unit)
			else
				icon = UnitCreatureType(unit)
			end
			icon = (icon and icons[icon]) or "UNKNOWN"
			classIcon:SetTexture("Interface\\AddOns\\SimpleUnitFrames\\Icons\\" .. icon .. ".tga")
			classIcon:Show()
		else
			classIcon:Hide()
		end
	end
end
